# Generated by Django 4.2.2 on 2024-04-30 07:57

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("axes", "0008_accessfailurelog"),
    ]

    operations = [
        migrations.AddField(
            model_name="accesslog",
            name="session_hash",
            field=models.CharField(
                blank=True,
                default="",
                max_length=64,
                verbose_name="Session key hash (sha256)",
            ),
        ),
    ]
